package news.chen.yu.cordova;

import android.util.Log;

import org.apache.cordova.*;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.tencent.mm.opensdk.modelmsg.SendAuth;
import com.tencent.mm.opensdk.modelpay.PayReq;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;

public class Wechat extends CordovaPlugin {
    public static String TAG = "ionic-cordova-plugins-wechat";
    public static String appKey;
    public static IWXAPI wxAPI;
    public static CallbackContext currentCallbackContext;

    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        Wechat.appKey = preferences.getString("wechat_app_id", "");
        Log.d(TAG, "Init: " + Wechat.appKey);
        Wechat.wxAPI = WXAPIFactory.createWXAPI(cordova.getActivity(), Wechat.appKey, true);
        Wechat.wxAPI.registerApp(Wechat.appKey);
    }

    @Override
    public boolean execute(String action, JSONArray args, final CallbackContext callbackContext) throws JSONException {
        Log.i(TAG, "Execute:" + action + " with :" + args.toString());
        if (action.equals("pay")) {
            JSONObject params = args.getJSONObject(0);
            this.sendPaymentRequest(params, callbackContext);
            return true;
        } else if(action.equals("auth")) {
            this.sendAuthRequest(args, callbackContext);
            return true;
        }
        return false;
    }

    protected boolean sendPaymentRequest(JSONObject params, CallbackContext callbackContext) {
        PayReq req = new PayReq();

        try {
            req.appId = Wechat.appKey;
            req.partnerId = params.has("mch_id") ? params.getString("mch_id") : params.getString("partnerid");
            req.prepayId = params.has("prepay_id") ? params.getString("prepay_id") : params.getString("prepayid");
            req.nonceStr = params.has("nonce") ? params.getString("nonce") : params.getString("noncestr");
            req.timeStamp = params.getString("timestamp");
            req.sign = params.getString("sign");
            req.packageValue = "Sign=WXPay";
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());

            callbackContext.error("参数错误");
            return true;
        }


        if (Wechat.wxAPI.sendReq(req)) {
            Log.i(TAG, "Payment request has been sent successfully.");

            // send no result
            sendNoResultPluginResult(callbackContext);
        } else {
            Log.i(TAG, "Payment request has been sent unsuccessfully.");

            // send error
            callbackContext.error("发送请求失败");
        }

        return true;
    }

    protected boolean sendAuthRequest(JSONArray args, CallbackContext callbackContext) {
        final SendAuth.Req req = new SendAuth.Req();
        try {
            req.scope = args.getString(0);
            req.state = args.getString(1);
        } catch (JSONException e) {
            Log.e(TAG, e.getMessage());

            req.scope = "snsapi_userinfo";
            req.state = "wechat";
        }

        if (Wechat.wxAPI.sendReq(req)) {
            Log.i(TAG, "Auth request has been sent successfully.");

            // send no result
            sendNoResultPluginResult(callbackContext);
        } else {
            Log.i(TAG, "Auth request has been sent unsuccessfully.");

            // send error
            callbackContext.error("发送请求失败");
        }

        return true;
    }

    private void sendNoResultPluginResult(CallbackContext callbackContext) {
        currentCallbackContext = callbackContext;

        PluginResult result = new PluginResult(PluginResult.Status.NO_RESULT);
        result.setKeepCallback(true);
        callbackContext.sendPluginResult(result);
    }
}